@@ -57,6 +57,23 @@ const config = {
     },
   ],
 
+  plugins: [
+    // ... Your other plugins.
+    [
+      require.resolve("docusaurus-plugin-search-local"),
+      {
+        // ... Your options.
+        // `hashed` is recommended as long-term-cache of index file is possible.
+        hashed: true,
+        // For Docs using Chinese, The `language` is recommended to set to:
+        // ```
+        // language: ["en", "zh"],
+        // ```
+        // When applying `zh` in language, please install `nodejieba` in your project.
+      },
+    ],
+  ],
+
   presets: [
     [
       'classic',
@@ -113,25 +130,15 @@ const config = {
         },
         items: [
           { to: '/docs/install', label: 'Install', position: 'left' },
-          {
-            to: '/docs/configuration-file',
-            label: 'Config',
-            position: 'left',
-          },
+          { to: '/docs/config', label: 'Config', position: 'left', },
           { to: '/docs/features', label: 'Features', position: 'left' },
+          { to: '/docs/releases', label: 'Releases', position: 'left' },
           { to: '/blog', label: 'Blog', position: 'left' },
-          { to: '/docs/changelog', label: 'Changelog', position: 'left' },
           {
             href: 'https://discord.gg/zRvJjmKGwS',
             label: 'Discord',
             position: 'left',
           },
-          // {
-          //   type: 'docsVersionDropdown',
-          //   position: 'right',
-          //   // dropdownItemsAfter: [{to: '/versions', label: 'All versions'}],
-          //   dropdownActiveClassDisabled: true,
-          // },
           {
             type: 'localeDropdown',
             position: 'right',
@@ -156,7 +163,7 @@ const config = {
               },
               {
                 label: 'Config',
-                to: '/docs/configuration-file',
+                to: '/docs/config',
               },
               {
                 label: 'Features',
