@@ -45,6 +45,7 @@ impl<T: rio_backend::event::EventListener> Drop for Context<T> {
 #[derive(Clone, Default)]
 pub struct ContextManagerConfig {
     pub shell: Shell,
+    #[cfg(not(target_os = "windows"))]
     pub use_fork: bool,
     pub working_dir: Option<String>,
     pub spawn_performer: bool,
@@ -137,12 +138,6 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
         size: SugarloafLayout,
         config: &ContextManagerConfig,
     ) -> Result<Context<T>, Box<dyn Error>> {
-        #[cfg(target_os = "windows")]
-        let width = size.width;
-
-        #[cfg(target_os = "windows")]
-        let height = size.height;
-
         let cols: u16 = size.columns.try_into().unwrap_or(MIN_COLUMNS as u16);
         let rows: u16 = size.lines.try_into().unwrap_or(MIN_LINES as u16);
 
@@ -197,13 +192,19 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
 
         #[cfg(target_os = "windows")]
         {
-            pty = create_pty(
+            pty = match create_pty(
                 &Cow::Borrowed(&config.shell.program),
                 config.shell.args.clone(),
                 &config.working_dir,
-                2,
-                1,
-            );
+                cols,
+                rows,
+            ) {
+                Ok(created_pty) => created_pty,
+                Err(err) => {
+                    log::error!("{err:?}");
+                    return Err(Box::new(err));
+                }
+            }
         }
 
         let machine = Machine::new(
@@ -220,15 +221,6 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
 
         let messenger = Messenger::new(channel);
 
-        #[cfg(target_os = "windows")]
-        {
-            if let Err(resize_error) =
-                messenger.send_resize(width as u16, height as u16, cols, rows)
-            {
-                log::error!("{resize_error:?}");
-            }
-        };
-
         Ok(Context {
             route_id,
             #[cfg(not(target_os = "windows"))]
@@ -322,6 +314,7 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
         window_id: WindowId,
     ) -> Result<Self, Box<dyn Error>> {
         let config = ContextManagerConfig {
+            #[cfg(not(target_os = "windows"))]
             use_fork: true,
             working_dir: None,
             shell: Shell {
@@ -566,7 +559,7 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
             if self.titles.last_title_update.elapsed() > Duration::from_secs(2) {
                 self.titles.last_title_update = Instant::now();
                 let mut id = String::from("");
-                for (i, context) in self.contexts.iter_mut().enumerate() {
+                for (i, _context) in self.contexts.iter().enumerate() {
                     let program = self.config.shell.program.to_owned();
                     let empty_string = String::from("");
 
@@ -695,10 +688,9 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
     ) {
         let mut working_dir = None;
         if self.config.use_current_path && self.config.working_dir.is_none() {
-            let current_context = self.current();
-
             #[cfg(not(target_os = "windows"))]
             {
+                let current_context = self.current();
                 if let Ok(path) = teletypewriter::foreground_process_path(
                     *current_context.main_fd,
                     current_context.shell_pid,
