@@ -4,12 +4,12 @@
 // See https://msdn.microsoft.com/en-us/library/4cc7ya5b.aspx for more details.
 #![windows_subsystem = "windows"]
 
-#[cfg(use_wa)]
-mod app;
+mod application;
 mod bindings;
 mod cli;
 mod constants;
 mod context;
+mod frame;
 mod ime;
 mod logger;
 mod messenger;
@@ -18,19 +18,16 @@ mod mouse;
 mod panic;
 mod platform;
 mod renderer;
-#[cfg(not(use_wa))]
 mod router;
 mod routes;
 mod scheduler;
-#[cfg(not(use_wa))]
 mod screen;
-#[cfg(not(use_wa))]
-mod sequencer;
 mod watcher;
 
 use clap::Parser;
 use log::{info, LevelFilter, SetLoggerError};
 use logger::Logger;
+use rio_backend::event::EventPayload;
 use rio_backend::{ansi, crosswords, event, performer, selection};
 use std::str::FromStr;
 
@@ -91,8 +88,7 @@ fn setup_logs_by_filter_level(log_level: &str) -> Result<(), SetLoggerError> {
     log::set_logger(&LOGGER).map(|()| log::set_max_level(filter_level))
 }
 
-#[tokio::main]
-async fn main() -> Result<(), Box<dyn std::error::Error>> {
+fn main() -> Result<(), Box<dyn std::error::Error>> {
     #[cfg(windows)]
     panic::attach_handler();
 
@@ -108,9 +104,9 @@ async fn main() -> Result<(), Box<dyn std::error::Error>> {
     let args = cli::Cli::parse();
 
     let write_config_path = args.window_options.terminal_options.write_config.clone();
-    if write_config_path.is_some() {
+    if let Some(config_path) = write_config_path {
         let _ = setup_logs_by_filter_level("TRACE");
-        rio_backend::config::create_config_file(write_config_path.unwrap());
+        rio_backend::config::create_config_file(config_path);
         return Ok(());
     }
 
@@ -145,18 +141,12 @@ async fn main() -> Result<(), Box<dyn std::error::Error>> {
 
     setup_environment_variables(&config);
 
-    #[cfg(not(use_wa))]
-    {
-        let window_event_loop = winit::event_loop::EventLoop::with_user_event()
-            .build()
-            .unwrap();
+    let window_event_loop =
+        rio_window::event_loop::EventLoop::<EventPayload>::with_user_event().build()?;
 
-        let mut sequencer = crate::sequencer::Sequencer::new(config, config_error);
-        let _ = sequencer.run(window_event_loop).await;
-    }
-
-    #[cfg(use_wa)]
-    let _ = app::run(config, config_error).await;
+    let mut application =
+        crate::application::Application::new(config, config_error, &window_event_loop);
+    let _ = application.run(window_event_loop);
 
     #[cfg(windows)]
     unsafe {
