@@ -1,21 +1,29 @@
 mod window;
 use crate::event::{EventPayload, EventProxy};
+use crate::frame::FrameTimer;
 use crate::router::window::{configure_window, create_window_builder};
 use crate::routes::{assistant, RoutePath};
+use crate::scheduler::TimerId;
+use crate::scheduler::{Scheduler, Topic};
 use crate::screen::{Screen, ScreenWindowProperties};
 use assistant::Assistant;
 use raw_window_handle::{HasDisplayHandle, HasWindowHandle};
+use rio_backend::clipboard::Clipboard;
 use rio_backend::config::Config as RioConfig;
 use rio_backend::error::{RioError, RioErrorLevel, RioErrorType};
-use std::collections::HashMap;
-use std::error::Error;
-use winit::event_loop::{ActiveEventLoop, EventLoop};
-use winit::keyboard::{Key, NamedKey};
+use rio_backend::event::RioEventType;
+use rio_window::event_loop::ActiveEventLoop;
+use rio_window::keyboard::{Key, NamedKey};
 #[cfg(not(any(target_os = "macos", windows)))]
-use winit::platform::startup_notify::{
+use rio_window::platform::startup_notify::{
     self, EventLoopExtStartupNotify, WindowAttributesExtStartupNotify,
 };
-use winit::window::{Window, WindowId};
+use rio_window::window::{Window, WindowId};
+use std::cell::RefCell;
+use std::collections::HashMap;
+use std::error::Error;
+use std::rc::Rc;
+use std::time::Duration;
 
 pub struct Route {
     pub assistant: assistant::Assistant,
@@ -41,10 +49,43 @@ impl Route {
 
 impl Route {
     #[inline]
-    pub fn redraw(&self) {
+    pub fn request_redraw(&mut self) {
         self.window.winit_window.request_redraw();
     }
 
+    /// Request a new frame for a window
+    pub fn request_frame(&mut self, scheduler: &mut Scheduler) {
+        // Mark that we've used a frame.
+        self.window.has_frame = false;
+
+        // Get the display vblank interval.
+        let monitor_vblank_interval = 1_000_000.
+            / self
+                .window
+                .winit_window
+                .current_monitor()
+                .and_then(|monitor| monitor.refresh_rate_millihertz())
+                .unwrap_or(60_000) as f64;
+
+        // Now convert it to micro seconds.
+        let monitor_vblank_interval =
+            Duration::from_micros((1000. * monitor_vblank_interval) as u64);
+
+        let swap_timeout = self
+            .window
+            .frame_timer
+            .compute_timeout(monitor_vblank_interval);
+
+        if swap_timeout > Duration::from_nanos(0) && !cfg!(feature = "wayland") {
+            let window_id = self.window.winit_window.id();
+            let timer_id = TimerId::new(Topic::Frame, window_id);
+            let event = EventPayload::new(RioEventType::Frame, window_id);
+            scheduler.schedule(event, swap_timeout, false, timer_id);
+        } else {
+            self.window.has_frame = true;
+        }
+    }
+
     #[inline]
     pub fn update_config(
         &mut self,
@@ -96,7 +137,7 @@ impl Route {
     }
 
     #[inline]
-    pub fn has_key_wait(&mut self, key_event: &winit::event::KeyEvent) -> bool {
+    pub fn has_key_wait(&mut self, key_event: &rio_window::event::KeyEvent) -> bool {
         if self.path == RoutePath::Terminal {
             return false;
         }
@@ -136,10 +177,14 @@ pub struct Router {
     propagated_report: Option<RioError>,
     pub font_library: Box<rio_backend::sugarloaf::font::FontLibrary>,
     pub config_route: Option<WindowId>,
+    pub clipboard: Rc<RefCell<Clipboard>>,
 }
 
 impl Router {
-    pub fn new(fonts: rio_backend::sugarloaf::font::SugarloafFonts) -> Router {
+    pub fn new(
+        fonts: rio_backend::sugarloaf::font::SugarloafFonts,
+        clipboard: Clipboard,
+    ) -> Router {
         let (font_library, fonts_not_found) =
             rio_backend::sugarloaf::font::FontLibrary::new(fonts);
 
@@ -152,11 +197,14 @@ impl Router {
             });
         }
 
+        let clipboard = Rc::new(RefCell::new(clipboard));
+
         Router {
             routes: HashMap::default(),
             propagated_report,
             config_route: None,
             font_library: Box::new(font_library),
+            clipboard,
         }
     }
 
@@ -220,6 +268,7 @@ impl Router {
             "Rio Settings",
             None,
             None,
+            self.clipboard.clone(),
         );
         let id = window.winit_window.id();
         let route = Route::new(Assistant::new(), RoutePath::Terminal, window);
@@ -243,6 +292,7 @@ impl Router {
             "Rio",
             None,
             open_url,
+            self.clipboard.clone(),
         );
         self.routes.insert(
             window.winit_window.id(),
@@ -272,6 +322,7 @@ impl Router {
             "Rio",
             tab_id,
             open_url,
+            self.clipboard.clone(),
         );
         self.routes.insert(
             window.winit_window.id(),
@@ -287,22 +338,24 @@ impl Router {
 pub struct RouteWindow {
     pub is_focused: bool,
     pub is_occluded: bool,
+    pub has_frame: bool,
+    pub has_updates: bool,
     pub winit_window: Window,
+    pub frame_timer: FrameTimer,
     pub screen: Screen<'static>,
     #[cfg(target_os = "macos")]
     pub is_macos_deadzone: bool,
 }
 
 impl RouteWindow {
-    pub async fn new(
-        event_loop: &EventLoop<EventPayload>,
+    pub fn new(
+        event_loop: &ActiveEventLoop,
+        event_proxy: &EventProxy,
         config: &rio_backend::config::Config,
         font_library: &rio_backend::sugarloaf::font::FontLibrary,
         open_url: Option<String>,
+        clipboard: &Rc<RefCell<Clipboard>>,
     ) -> Result<RouteWindow, Box<dyn Error>> {
-        let proxy = event_loop.create_proxy();
-        let event_proxy = EventProxy::new(proxy.clone());
-
         #[allow(unused_mut)]
         let mut window_builder = create_window_builder("Rio", config, None);
 
@@ -319,12 +372,21 @@ impl RouteWindow {
             theme: winit_window.theme(),
         };
 
-        let screen =
-            Screen::new(properties, config, event_proxy, font_library, open_url).await?;
+        let screen = Screen::new(
+            properties,
+            config,
+            event_proxy.clone(),
+            font_library,
+            open_url,
+            clipboard.clone(),
+        )?;
 
         Ok(Self {
             is_focused: false,
             is_occluded: false,
+            has_frame: true,
+            has_updates: true,
+            frame_timer: FrameTimer::new(),
             winit_window,
             screen,
             #[cfg(target_os = "macos")]
@@ -332,6 +394,7 @@ impl RouteWindow {
         })
     }
 
+    #[allow(clippy::too_many_arguments)]
     pub fn from_target(
         event_loop: &ActiveEventLoop,
         event_proxy: EventProxy,
@@ -340,6 +403,7 @@ impl RouteWindow {
         window_name: &str,
         tab_id: Option<String>,
         open_url: Option<String>,
+        clipboard: Rc<RefCell<Clipboard>>,
     ) -> RouteWindow {
         #[allow(unused_mut)]
         let mut window_builder =
@@ -367,16 +431,20 @@ impl RouteWindow {
             theme: winit_window.theme(),
         };
 
-        let screen = futures::executor::block_on(Screen::new(
+        let screen = Screen::new(
             properties,
             config,
             event_proxy,
             font_library,
             open_url,
-        ))
+            clipboard,
+        )
         .expect("Screen not created");
 
         Self {
+            has_frame: true,
+            has_updates: true,
+            frame_timer: FrameTimer::new(),
             is_focused: false,
             is_occluded: false,
             winit_window,
