@@ -14,7 +14,7 @@ use crate::bindings::{
     ViAction,
 };
 #[cfg(target_os = "macos")]
-use crate::constants::{DEADZONE_END_Y, DEADZONE_START_X, DEADZONE_START_Y};
+use crate::constants::{DEADZONE_END_Y, DEADZONE_START_Y};
 use crate::context::{self, process_open_url, ContextManager};
 use crate::crosswords::{
     grid::{Dimensions, Scroll},
@@ -33,7 +33,8 @@ use crate::screen::hint::HintMatches;
 use crate::selection::{Selection, SelectionType};
 use core::fmt::Debug;
 use raw_window_handle::{RawDisplayHandle, RawWindowHandle};
-use rio_backend::clipboard::{Clipboard, ClipboardType};
+use rio_backend::clipboard::Clipboard;
+use rio_backend::clipboard::ClipboardType;
 use rio_backend::config::{
     colors::term::List,
     renderer::{Backend as RendererBackend, Performance as RendererPerformance},
@@ -45,18 +46,20 @@ use rio_backend::sugarloaf::{
     layout::SugarloafLayout, Sugarloaf, SugarloafErrors, SugarloafRenderer,
     SugarloafWindow, SugarloafWindowSize,
 };
+use rio_window::event::ElementState;
+use rio_window::event::Modifiers;
+use rio_window::event::MouseButton;
+#[cfg(target_os = "macos")]
+use rio_window::keyboard::ModifiersKeyState;
+use rio_window::keyboard::{Key, KeyLocation, ModifiersState, NamedKey};
+use rio_window::platform::modifier_supplement::KeyEventExtModifierSupplement;
 use std::borrow::Cow;
+use std::cell::RefCell;
 use std::cmp::{max, min};
 use std::error::Error;
 use std::ffi::OsStr;
+use std::rc::Rc;
 use touch::TouchPurpose;
-use winit::event::ElementState;
-use winit::event::Modifiers;
-use winit::event::MouseButton;
-#[cfg(target_os = "macos")]
-use winit::keyboard::ModifiersKeyState;
-use winit::keyboard::{Key, KeyLocation, ModifiersState, NamedKey};
-use winit::platform::modifier_supplement::KeyEventExtModifierSupplement;
 
 /// Minimum number of pixels at the bottom/top where selection scrolling is performed.
 const MIN_SELECTION_SCROLLING_HEIGHT: f32 = 5.;
@@ -73,7 +76,6 @@ const MAX_SEARCH_HISTORY_SIZE: usize = 255;
 pub struct Screen<'screen> {
     bindings: crate::bindings::KeyBindings,
     mouse_bindings: Vec<MouseBinding>,
-    clipboard: Clipboard,
     pub modifiers: Modifiers,
     pub mouse: Mouse,
     pub touchpurpose: TouchPurpose,
@@ -82,24 +84,26 @@ pub struct Screen<'screen> {
     pub renderer: Renderer,
     pub sugarloaf: Sugarloaf<'screen>,
     pub context_manager: context::ContextManager<EventProxy>,
+    pub clipboard: Rc<RefCell<Clipboard>>,
 }
 
 pub struct ScreenWindowProperties {
-    pub size: winit::dpi::PhysicalSize<u32>,
+    pub size: rio_window::dpi::PhysicalSize<u32>,
     pub scale: f64,
     pub raw_window_handle: RawWindowHandle,
     pub raw_display_handle: RawDisplayHandle,
-    pub window_id: winit::window::WindowId,
-    pub theme: Option<winit::window::Theme>,
+    pub window_id: rio_window::window::WindowId,
+    pub theme: Option<rio_window::window::Theme>,
 }
 
 impl Screen<'_> {
-    pub async fn new<'screen>(
+    pub fn new<'screen>(
         window_properties: ScreenWindowProperties,
         config: &rio_backend::config::Config,
         event_proxy: EventProxy,
         font_library: &rio_backend::sugarloaf::font::FontLibrary,
         open_url: Option<String>,
+        clipboard: Rc<RefCell<Clipboard>>,
     ) -> Result<Screen<'screen>, Box<dyn Error>> {
         let size = window_properties.size;
         let scale = window_properties.scale;
@@ -165,9 +169,7 @@ impl Screen<'_> {
             sugarloaf_renderer,
             font_library,
             sugarloaf_layout,
-        )
-        .await
-        {
+        ) {
             Ok(instance) => instance,
             Err(instance_with_errors) => {
                 sugarloaf_errors = Some(instance_with_errors.errors);
@@ -177,9 +179,6 @@ impl Screen<'_> {
 
         let renderer = Renderer::new(config, theme);
 
-        // let clipboard = unsafe { Clipboard::new(raw_display_handle.into()) };
-        let clipboard = unsafe { Clipboard::new(raw_display_handle) };
-
         let bindings = crate::bindings::default_key_bindings(
             config.bindings.keys.to_owned(),
             config.navigation.has_navigation_key_bindings(),
@@ -202,6 +201,7 @@ impl Screen<'_> {
             shell,
             working_dir,
             spawn_performer: true,
+            #[cfg(not(target_os = "windows"))]
             use_fork: config.use_fork,
             is_native,
             // When navigation is collapsed and does not contain any color rule
@@ -282,6 +282,7 @@ impl Screen<'_> {
         )
     }
 
+    #[inline]
     pub fn touch_purpose(&mut self) -> &mut TouchPurpose {
         &mut self.touchpurpose
     }
@@ -294,20 +295,12 @@ impl Screen<'_> {
         pos_y <= DEADZONE_START_Y * scale_f64 && pos_y >= DEADZONE_END_Y * scale_f64
     }
 
-    #[inline]
-    #[cfg(target_os = "macos")]
-    pub fn is_macos_deadzone_draggable(&self, pos_x: f64) -> bool {
-        let layout = self.sugarloaf.layout();
-        let scale_f64 = layout.dimensions.scale as f64;
-        pos_x >= DEADZONE_START_X * scale_f64
-    }
-
     /// update_config is triggered in any configuration file update
     #[inline]
     pub fn update_config(
         &mut self,
         config: &rio_backend::config::Config,
-        current_theme: Option<winit::window::Theme>,
+        current_theme: Option<rio_window::window::Theme>,
         font_library: &rio_backend::sugarloaf::font::FontLibrary,
     ) {
         let num_tabs = self.ctx().len();
@@ -338,6 +331,7 @@ impl Screen<'_> {
             terminal.cursor_shape = cursor;
             terminal.default_cursor_shape = cursor;
             terminal.blinking_cursor = config.blinking_cursor;
+            drop(terminal);
         }
 
         self.mouse
@@ -368,7 +362,10 @@ impl Screen<'_> {
     }
 
     #[inline]
-    pub fn resize(&mut self, new_size: winit::dpi::PhysicalSize<u32>) -> &mut Self {
+    pub fn resize(&mut self, new_size: rio_window::dpi::PhysicalSize<u32>) -> &mut Self {
+        if self.renderer.selection_range.is_some() {
+            self.clear_selection();
+        }
         self.sugarloaf.resize(new_size.width, new_size.height);
         self.resize_all_contexts();
         self
@@ -378,7 +375,7 @@ impl Screen<'_> {
     pub fn set_scale(
         &mut self,
         new_scale: f32,
-        new_size: winit::dpi::PhysicalSize<u32>,
+        new_size: rio_window::dpi::PhysicalSize<u32>,
     ) -> &mut Self {
         self.sugarloaf.rescale(new_scale);
         self.sugarloaf.resize(new_size.width, new_size.height);
@@ -402,25 +399,11 @@ impl Screen<'_> {
             let mut terminal = context.terminal.lock();
             terminal.resize::<SugarloafLayout>(layout);
             drop(terminal);
-            let _ = context.messenger.send_resize(
-                layout.width as u16,
-                layout.height as u16,
-                layout.columns as u16,
-                layout.lines as u16,
-            );
+            let winsize = crate::renderer::utils::terminal_dimensions(&layout);
+            let _ = context.messenger.send_resize(winsize);
         }
     }
 
-    #[inline]
-    pub fn clipboard_get(&mut self, clipboard_type: ClipboardType) -> String {
-        self.clipboard.get(clipboard_type)
-    }
-
-    #[inline]
-    pub fn clipboard_store(&mut self, clipboard_type: ClipboardType, content: String) {
-        self.clipboard.set(clipboard_type, content);
-    }
-
     #[inline]
     pub fn scroll_bottom_when_cursor_not_visible(&mut self) {
         let mut terminal = self.ctx_mut().current_mut().terminal.lock();
@@ -462,7 +445,7 @@ impl Screen<'_> {
     }
 
     #[inline]
-    pub fn process_key_event(&mut self, key: &winit::event::KeyEvent) {
+    pub fn process_key_event(&mut self, key: &rio_window::event::KeyEvent) {
         // 1. In case there is a key released event and Rio is not using kitty keyboard protocol
         // then should return drop the key processing
         // 2. In case IME has preedit then also should drop the key processing
@@ -610,7 +593,7 @@ impl Screen<'_> {
             if binding.is_triggered_by(binding_mode.to_owned(), mods, &button)
                 && binding.action == Act::PasteSelection
             {
-                let content = self.clipboard.get(ClipboardType::Selection);
+                let content = self.clipboard.borrow_mut().get(ClipboardType::Selection);
                 self.paste(&content, true);
             }
         }
@@ -618,7 +601,7 @@ impl Screen<'_> {
 
     pub fn process_key_bindings(
         &mut self,
-        key: &winit::event::KeyEvent,
+        key: &rio_window::event::KeyEvent,
         mode: &Mode,
         mods: ModifiersState,
     ) -> bool {
@@ -678,14 +661,16 @@ impl Screen<'_> {
                             .send_bytes(s.to_owned().into_bytes());
                     }
                     Act::Paste => {
-                        let content = self.clipboard.get(ClipboardType::Clipboard);
+                        let content =
+                            self.clipboard.borrow_mut().get(ClipboardType::Clipboard);
                         self.paste(&content, true);
                     }
                     Act::ClearSelection => {
                         self.clear_selection();
                     }
                     Act::PasteSelection => {
-                        let content = self.clipboard.get(ClipboardType::Selection);
+                        let content =
+                            self.clipboard.borrow_mut().get(ClipboardType::Selection);
                         self.paste(&content, true);
                     }
                     Act::Copy => {
@@ -1061,6 +1046,7 @@ impl Screen<'_> {
             };
 
             terminal.scroll_to_pos(new_origin);
+            drop(terminal);
 
             self.search_state.display_offset_delta = 0;
             self.search_state.origin = new_origin;
@@ -1096,11 +1082,12 @@ impl Screen<'_> {
 
         // Store origin and scroll back to the match.
         terminal.scroll_display(Scroll::Delta(-self.search_state.display_offset_delta));
+        drop(terminal);
         self.search_state.origin = new_origin;
     }
 
     /// Whether we should send `ESC` due to `Alt` being pressed.
-    fn alt_send_esc(&mut self, key: &winit::event::KeyEvent, text: &str) -> bool {
+    fn alt_send_esc(&mut self, key: &rio_window::event::KeyEvent, text: &str) -> bool {
         #[cfg(not(target_os = "macos"))]
         let alt_send_esc = self.modifiers.state().alt_key();
 
@@ -1137,9 +1124,11 @@ impl Screen<'_> {
         drop(terminal);
 
         if ty == ClipboardType::Selection {
-            self.clipboard.set(ClipboardType::Clipboard, text.clone());
+            self.clipboard
+                .borrow_mut()
+                .set(ClipboardType::Clipboard, text.clone());
         }
-        self.clipboard.set(ty, text);
+        self.clipboard.borrow_mut().set(ty, text);
     }
 
     #[inline]
@@ -1584,6 +1573,7 @@ impl Screen<'_> {
             terminal.vi_mode_cursor.pos = self.search_state.origin;
             terminal
                 .scroll_display(Scroll::Delta(self.search_state.display_offset_delta));
+            drop(terminal);
         }
         self.search_state.display_offset_delta = 0;
         self.sugarloaf.mark_dirty();
@@ -1649,6 +1639,7 @@ impl Screen<'_> {
                     self.search_state.focused_match = None;
                 }
             }
+            drop(terminal);
         }
 
         if should_reset_search_state {
