@@ -1,19 +1,17 @@
 extern crate png;
-extern crate tokio;
 
 use raw_window_handle::{HasDisplayHandle, HasWindowHandle};
+use rio_window::event::WindowEvent;
+use rio_window::event_loop::ControlFlow;
+use rio_window::platform::run_on_demand::EventLoopExtRunOnDemand;
+use rio_window::{
+    dpi::LogicalSize, event::Event, event_loop::EventLoop, window::WindowAttributes,
+};
 use sugarloaf::components::rect::Rect;
 use sugarloaf::layout::SugarloafLayout;
 use sugarloaf::{Object, Sugarloaf, SugarloafWindow, SugarloafWindowSize};
-use winit::event::WindowEvent;
-use winit::event_loop::ControlFlow;
-use winit::platform::run_on_demand::EventLoopExtRunOnDemand;
-use winit::{
-    dpi::LogicalSize, event::Event, event_loop::EventLoop, window::WindowAttributes,
-};
 
-#[tokio::main]
-async fn main() {
+fn main() {
     let mut event_loop = EventLoop::new().unwrap();
     let width = 400.0;
     let height = 200.0;
@@ -56,7 +54,6 @@ async fn main() {
         &sugarloaf::font::FontLibrary::default(),
         sugarloaf_layout,
     )
-    .await
     .expect("Sugarloaf instance should be created");
 
     sugarloaf.set_background_color(wgpu::Color::TRANSPARENT);
